#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
Nada Player Menu
"""

import curses
import locale
import sys
import time

from ui import UI
from player import Player
from api import Luoo
from database import Database
from downloader import Downloader
from .common import *

locale.setlocale(locale.LC_ALL, "")
code = locale.getpreferredencoding()

if os.path.isdir(BASE_PATH) is False:
    os.mkdir(BASE_PATH)


class Menu:
    def __init__(self):
        reload(sys)
        sys.setdefaultencoding('UTF-8')

        self.title = 'Nada'
        self.model = ['最新期刊', '分类期刊', '搜索期刊', '收藏夹', '关于']
        self.view = 'menu'
        self.ctrl = 'menu'

        self.offset = 0
        self.index = 0
        self.step = 10
        self.play_id = -1
        self.play_vol = -1

        self.present = []
        self.stack = []

        self.player = Player()
        self.ui = UI()
        self.luoo = Luoo()
        self.downloader = Downloader()

        self.database = Database()
        self.database.load()
        self.collections = self.database.data['collections'][0]

        self.screen = curses.initscr()
        self.screen.keypad(1)

    def start(self):
        self.ui.build(self.title, self.model, self.view, self.offset, self.index, self.step, self.play_vol,
                      self.play_id)
        self.stack.append([self.title, self.model, self.view, self.ctrl, self.offset, self.index])

        while True:
            title = self.title
            model = self.model
            view = self.view
            ctrl = self.ctrl

            offset = self.offset
            index = idx = self.index
            step = self.step

            key = self.screen.getch()
            self.ui.screen.refresh()

            if view == 'songs':
                length = len(model['songs'])
            else:
                length = len(model)

            if key == ord('q'):
                break

            elif key == ord('k'):
                if idx == offset:
                    if offset == 0:
                        continue
                    self.offset -= step
                    self.index = offset - 1
                else:
                    self.index = carousel(offset, min(length, offset + step) - 1, idx - 1)

            elif key == ord('j'):
                if idx == min(length, offset + step) - 1:
                    if offset + step >= length:
                        continue
                    self.offset += step
                    self.index = offset + step
                else:
                    self.index = carousel(offset, min(length, offset + step) - 1, idx + 1)

            elif key == ord('u'):
                if offset == 0:
                    continue
                self.offset -= step
                self.index = (index - step) // step * step

            elif key == ord('i'):
                if offset + step >= length:
                    continue
                self.offset += step
                self.index = (index + step) // step * step

            elif key == ord('l'):
                if self.view == 'songs':
                    continue
                self.ui.loading()
                self.control(idx)
                self.index = 0
                self.offset = 0

            elif key == ord('h'):
                if len(self.stack) == 1:
                    continue
                up = self.stack.pop()
                self.title = up[0]
                self.model = up[1]
                self.view = up[2]
                self.ctrl = up[3]
                self.offset = up[4]
                self.index = up[5]

            elif key == ord(' '):
                if view == 'songs':
                    self.present = [title, model, view, ctrl, offset]
                self.player.play_song(view, model, idx)

            elif key == ord(']'):
                self.player.next_song()
                if view == 'songs':
                    self.index = self.player.play_id % length
                    if idx == min(length, offset + step) - 1:
                        if offset + step >= length:
                            continue
                        self.offset += step
                time.sleep(0.1)

            elif key == ord('['):
                self.player.prev_song()
                if view == 'songs':
                    self.index = self.player.play_id % length
                    if idx == offset:
                        if offset == 0:
                            continue
                        self.offset -= step
                time.sleep(0.1)

            elif key == ord('p'):
                if len(self.present) == 0:
                    continue
                self.stack.append([self.title, self.model, self.view, self.ctrl, self.offset, self.index])

                self.title = self.present[0]
                self.model = self.present[1]
                self.view = self.present[2]
                self.ctrl = self.present[3]
                self.index = idx = self.player.play_id
                self.offset = idx // step * step

            elif key == ord('m'):
                if view != 'menu':
                    self.stack.append([self.title, self.model, self.view, self.ctrl, self.offset, self.index])

                    self.title = self.stack[0][0]
                    self.model = self.stack[0][1]
                    self.view = self.stack[0][2]
                    self.ctrl = self.stack[0][3]
                    self.offset = self.stack[0][4]
                    self.index = self.stack[0][5]

            elif key == ord('a'):
                if view == 'songs' and ctrl is not 'collections':
                    self.ui.status(model['songs'][idx]['name'], 'Add', 5)
                    self.collections.append(model['songs'][idx])

            elif key == ord('r'):
                if ctrl == 'collections':
                    if length != 0:
                        self.ui.status(model['songs'][idx]['name'], 'Remove', 3)
                        self.model['songs'].pop(idx)
                        self.index = carousel(offset, min(length, offset + step) - 1, idx)

            elif key == ord('c'):
                self.stack.append([self.title, self.model, self.view, self.ctrl, self.offset, self.index])
                self.title = 'Nada > nada 收藏'
                self.collection()

            elif key == ord('d'):
                if view == 'songs':
                    self.downloader.download(model['songs'][idx])

            self.play_vol = self.player.play_vol
            self.play_id = self.player.play_id
            self.ui.build(self.title, self.model, self.view, self.offset, self.index, self.step, self.play_vol,
                          self.play_id)
            self.ui.screen.refresh()

        self.player.stop()
        self.database.save()
        curses.endwin()

    def control(self, idx):
        self.stack.append([self.title, self.model, self.view, self.ctrl, self.offset, self.index])

        ctrl = self.ctrl
        if ctrl == 'menu':
            self.menu(idx)

        elif ctrl == 'vols':
            self.vols(idx)

        elif ctrl == 'vtype':
            self.vtype(idx)

        self.offset = 0
        self.index = 0

    def menu(self, idx):
        if idx == 0:
            self.title += ' > 最新期刊'
            self.model = self.luoo.new()
            self.view = 'vols'
            self.ctrl = 'vols'

        elif idx == 1:
            self.title += ' > 分类期刊'
            self.model = self.luoo.vtype()
            self.view = 'vols'
            self.ctrl = 'vtype'

        elif idx == 2:
            vol_id = self.ui.search('搜索期刊: ')
            try:
                self.model = self.luoo.vol(vol_id)
                self.title += ' > 搜索期刊 > Vol.' + self.model['id'] + ' ' + self.model['title']
                self.view = 'songs'
                self.ctrl = 'songs'
            except Exception, e:
                self.model = []

        elif idx == 3:
            self.title += ' > 收藏夹'
            self.collection()

        elif idx == 4:
            self.title += ' > 关于'
            self.view = 'about'
            self.ctrl = 'about'

    def collection(self):
        self.model = {'id': 'collections', 'songs': self.collections}
        self.view = 'songs'
        self.ctrl = 'collections'
        self.offset = 0
        self.index = 0

    def vols(self, idx):
        self.title += ' > ' + self.model[idx]['name']
        vol_id = self.model[idx]['id']
        self.model = self.luoo.vol(vol_id)
        self.view = 'songs'
        self.ctrl = 'songs'

    def vtype(self, idx):
        self.title += ' > ' + self.model[idx]['name']
        type_id = self.model[idx]['id']
        self.model = self.luoo.vols(type_id)
        self.view = 'vols'
        self.ctrl = 'vols'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Singleton Pattern
"""

class Singleton(object):  
    def __new__(cls, *args, **kw):  
        if not hasattr(cls, '_instance'):  
            orig = super(Singleton, cls)  
            cls._instance = orig.__new__(cls, *args, **kw)  
        return cls._instance  



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import curses
import time


class UI:
    def __init__(self):
        self.screen = curses.initscr()
        curses.cbreak()
        curses.noecho()
        self.screen.keypad(1)

        curses.start_color()
        curses.init_pair(1, curses.COLOR_GREEN, curses.COLOR_BLACK)
        curses.init_pair(2, curses.COLOR_CYAN, curses.COLOR_BLACK)
        curses.init_pair(3, curses.COLOR_RED, curses.COLOR_BLACK)
        curses.init_pair(4, curses.COLOR_YELLOW, curses.COLOR_BLACK)
        curses.init_pair(5, curses.COLOR_BLUE, curses.COLOR_BLACK)

    def playinfo(self, song, pause=False):
        self.screen.move(1, 1)
        self.screen.clrtoeol()
        self.screen.move(2, 1)
        self.screen.clrtoeol()
        if pause:
            self.screen.addstr(2, 5, '_ - z Z Z', curses.color_pair(3))
        else:
            self.screen.addstr(2, 5, '♩  ♪  ♫  ♬', curses.color_pair(3))
        self.screen.addstr(2, 19, song['name'] + ' - ' + song['artist'], curses.color_pair(4))
        self.screen.refresh()

    def status(self, info, opt, color=1):
        string = opt + ' <' + info + '>'
        self.screen.move(20, 1)
        self.screen.clrtoeol()
        self.screen.addstr(20, 5, string, curses.color_pair(color))
        self.screen.refresh()
        time.sleep(0.5)
        self.screen.move(20, 1)
        self.screen.clrtoeol()

    def loading(self):
        self.screen.addstr(6, 5, '♫  ♪ Nada ♫  ♪ Loading...', curses.color_pair(1))
        self.screen.refresh()

    def build(self, title, model, view, offset, index, step, play_vol, play_id):
        self.screen.move(4, 1)
        self.screen.clrtobot()
        self.screen.addstr(5, 5, title, curses.color_pair(1))

        if view == 'songs':
            model_len = len(model['songs'])
        else:
            model_len = len(model)

        if model_len == 0:
            self.screen.addstr(8, 5, 'Nothing ...')
        else:
            if view == 'menu':
                for i in xrange(offset, min(len(model), offset + step)):
                    if i == index:
                        self.screen.addstr(i - offset + 8, 2, '-> ' + model[i], curses.color_pair(2))
                    else:
                        self.screen.addstr(i - offset + 8, 5, model[i])

            elif view == 'list':
                for i in xrange(offset, min(len(model), offset + step)):
                    if i == index:
                        self.screen.addstr(i - offset + 8, 2, '-> ' + model[i], curses.color_pair(2))
                    else:
                        self.screen.addstr(i - offset + 8, 5, model[i])

            elif view == 'vols':
                for i in xrange(offset, min(len(model), offset + step)):
                    if i == index:
                        self.screen.addstr(i - offset + 8, 2, '-> ' + model[i]["name"], curses.color_pair(2))
                    else:
                        self.screen.addstr(i - offset + 8, 5, model[i]["name"])

            elif view == 'songs':
                vol_id = model['id']
                songs = model['songs']
                for i in xrange(offset, min(len(songs), offset + step)):
                    if i == index and i == play_id and play_vol == vol_id:
                        self.screen.addstr(i - offset + 8, 2, '>> ' + str(i) + '. ' + songs[i]['name'] + ' - ' + songs[i]['artist'],
                                           curses.color_pair(2))
                    elif i == play_id and play_vol == vol_id:
                        self.screen.addstr(i - offset + 8, 3, '> ' + str(i) + '. ' + songs[i]['name'] + ' - ' + songs[i]['artist'],
                                           curses.color_pair(5))
                    elif i == index:
                        self.screen.addstr(i - offset + 8, 2, '-> ' + str(i) + '. ' + songs[i]['name'] + ' - ' + songs[i]['artist'],
                                           curses.color_pair(2))
                    else:
                        self.screen.addstr(i - offset + 8, 5, str(i) + '. ' + songs[i]['name'] + ' - ' + songs[i]['artist'])

            elif view == 'about':
                self.screen.addstr(8, 12, 'The mind is like a serpent, ')
                self.screen.addstr(9, 12, 'forgetting all its unsteadiness by hearing the nada,')
                self.screen.addstr(10, 12, 'it does not run away anywhere.')
                self.screen.addstr(12, 45, 'By Ahonn@Nada-1.0.0')

    def search(self, string):
        self.screen.move(4, 1)
        self.screen.clrtobot()
        self.screen.addstr(5, 19, string, curses.color_pair(1))
        self.screen.refresh()
        curses.echo()
        info = self.screen.getstr(10, 10, 60)
        curses.noecho()
        if info.strip() is '':
            return self.search(string)
        else:
            return info



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import signal
import subprocess
import threading
import time

from ui import UI
from .common import *


class Player:
    def __init__(self):
        self.ui = UI()

        self.popen_handler = None
        self.play = False
        self.pause = False
        self.songs = []
        self.play_vol = -1
        self.play_id = -1
        self.view = 'songs'

    def popen_recall(self, onExit, popenArgs):
        def runInThread(onExit, popenArgs):
            self.popen_handler = subprocess.Popen(['mpg123', popenArgs], stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                                  stderr=subprocess.PIPE)
            self.popen_handler.wait()
            if self.play:
                self.play_id = carousel(0, len(self.songs) - 1, self.play_id + 1)
                onExit()
            return

        thread = threading.Thread(target=runInThread, args=(onExit, popenArgs))
        thread.start()
        return thread

    def recall(self):
        self.play = True
        song = self.songs[self.play_id]
        self.ui.playinfo(song)
        self.popen_recall(self.recall, song['source'])

    def play_song(self, view, model, idx):
        self.view = view

        if view == 'songs':
            songs = model['songs']
            id = model['id']

            if idx == self.play_id and songs == self.songs:
                if self.pause:
                    self.resume()
                else:
                    self.pause_song()
            else:
                self.songs = songs
                self.play_id = idx
                self.play_vol = id

                if self.play:
                    self.switch()
                else:
                    self.recall()

        else:
            if self.play:
                if self.pause:
                    self.resume()
                else:
                    self.pause_song()

    def pause_song(self):
        self.pause = True
        os.kill(self.popen_handler.pid, signal.SIGSTOP)
        self.ui.playinfo(self.songs[self.play_id], pause=True)

    def resume(self):
        self.pause = False
        os.kill(self.popen_handler.pid, signal.SIGCONT)
        self.ui.playinfo(self.songs[self.play_id])

    def switch(self):
        self.stop()
        time.sleep(0.1)
        self.recall()

    def stop(self):
        if self.play and self.popen_handler:
            self.popen_handler.kill()
            self.play = False

    def next_song(self):
        self.stop()
        time.sleep(0.1)
        self.play_id = carousel(0, len(self.songs) - 1, self.play_id + 1)
        self.recall()

    def prev_song(self):
        self.stop()
        time.sleep(0.1)
        self.play_id = carousel(0, len(self.songs) - 1, self.play_id - 1)
        self.recall()



#!/usr/bin/env python
# encoding: UTF-8

from setuptools import setup, find_packages

setup(
    name='nada',
    version='1.0.1',
    keywords=['music', 'luoo', 'cli', 'player'],
    description="Luoo music command line player",
    license='MIT License',
    packages=find_packages(),

    include_package_data=True,

    install_requires=[
        'requests',
        'BeautifulSoup4',
        'lxml'
    ],

    entry_points={
        'console_scripts': [
            'nada = nada:start'
        ],
    },

    author='ahonn',
    author_email='ahonn95@outlook.com',
    url='https://github.com/ahonn/Nada',
    zip_safe=False,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import json
import requests
from bs4 import BeautifulSoup


class Luoo:
    def __init__(self):
        self.url = 'http://www.luoo.net'

    def parser(self, url):
        request = requests.get(url, timeout=5)
        return BeautifulSoup(request.content, "lxml")

    def new(self):
        return self.vols()

    def vols(self, vtype=''):
        vols = []
        for page in xrange(1, 10):
            url = self.url + '/tag/' + vtype + '?p=' + str(page)
            soup = self.parser(url)

            items = soup.find_all("a", class_="name")
            for item in items:
                vols.append({
                    "id": item["href"].split('/')[-1],
                    "name": item.string.encode("utf-8")
                })
        return vols

    def vol(self, id):
        url = self.url + '/music/' + id
        soup = self.parser(url)

        title = soup.find("span", class_="vol-title").text
        vol = {'id': id, 'title': title, 'songs': []}

        items = soup.find_all("li", class_="track-item")
        for i, item in enumerate(items):
            vol['songs'].append({
                "name": item.find("a", class_="trackname").text.split(' ', 1)[1],
                "artist": item.find("span", class_="artist").text,
                "source": 'http://luoo-mp3.kssws.ks-cdn.com/low/luoo/radio' + id + '/' + str(
                    "%02d" % (i + 1)) + '.mp3'
            })
        return vol

    def vtype(self):
        url = self.url + '/music/'
        soup = self.parser(url)

        vtype = []
        items = soup.find_all("a", class_="item")
        for item in items:
            if item.text == '最新期刊':
                continue
            vtype.append({
                "name": item.text,
                "id": item["href"].split('/')[-1]
            })
        return vtype
